@@ -1,6 +1,8 @@
 """Library to handle connection with mill."""
 # Based on https://pastebin.com/53Nk0wJA and Postman capturing from the app
 import asyncio
+from base64 import b64encode
+from dataclasses import dataclass
 import datetime as dt
 import hashlib
 import json
@@ -8,7 +10,6 @@ import logging
 import random
 import string
 import time
-from base64 import b64encode
 
 import aiohttp
 import async_timeout
@@ -21,7 +22,7 @@ API_ENDPOINT_2 = "https://eurouter.ablecloud.cn:9005/millService/v1/"
 API_ENDPOINT_STATS = "https://api.millheat.com/statistics/"
 DEFAULT_TIMEOUT = 10
 MIN_TIME_BETWEEN_UPDATES = dt.timedelta(seconds=5)
-MIN_TIME_BETWEEN_STATS_UPDATES = dt.timedelta(minutes=30)
+MIN_TIME_BETWEEN_STATS_UPDATES = dt.timedelta(minutes=10)
 REQUEST_TIMEOUT = "300"
 
 _LOGGER = logging.getLogger(__name__)
@@ -50,9 +51,9 @@ class Mill:
         self._token = None
         self.rooms = {}
         self.heaters = {}
+        self.sensors = {}
         self._throttle_time = None
         self._throttle_all_time = None
-        self._lock_cons_data = asyncio.Lock()
 
         key = (
             b"vO\xe4O\xe0G\xeb|$\x9d\x8375\xd6\x1bl\xca\x96'\x8f"
@@ -261,6 +262,7 @@ class Mill:
                 self.rooms[_id] = room
                 payload = {"roomId": _room.get("roomId"), "timeZoneNum": "+01:00"}
                 room_device = await self.request("selectDevicebyRoom", payload)
+                # room_device = await self.request("selectDevicebyRoom2020", payload)
 
                 room.always = room_device.get("always")
                 room.backHour = room_device.get("backHour")
@@ -288,7 +290,7 @@ class Mill:
         if sleep_temp is None and comfort_temp is None and away_temp is None:
             return
         for room_id, _room in self.rooms.items():
-            if _room.name == room_name:
+            if _room.name.lower().strip() == room_name.lower().strip():
                 await self.set_room_temperatures(
                     room_id, sleep_temp, comfort_temp, away_temp
                 )
@@ -313,9 +315,8 @@ class Mill:
             "sleepTemp": room.sleep_temp,
             "comfortTemp": room.comfort_temp,
             "awayTemp": room.away_temp,
-            "homeType": 0,
         }
-        await self.request("changeRoomModeTempInfo", payload)
+        await self.request("changeRoomModeTempInfoAway", payload)
         self.rooms[room_id] = room
 
     async def set_room_mode(self, room_id, mode=None, hour=0, minute=0):
@@ -360,12 +361,20 @@ class Mill:
         await self.update_heaters()
         return self.heaters
 
+    async def fetch_heater_sensor_data(self):
+        """Request data."""
+        if not self.heaters:
+            await self.update_rooms()
+
+        await self.update_heaters()
+        return {**self.heaters, **self.sensors}
+
     async def update_heaters(self):
         """Request data."""
         tasks = []
         homes = await self.get_home_list()
         for home in homes:
-            tasks.append(self._update_heater_home_data(home))
+            tasks.append(self._update_independent_devices(home))
 
         for heater in self.heaters.values():
             tasks.append(self._update_consumption(heater))
@@ -374,22 +383,27 @@ class Mill:
             tasks.append(self._update_heater_data(heater))
         await asyncio.gather(*tasks)
 
-    async def _update_heater_home_data(self, home):
+    async def _update_independent_devices(self, home):
         payload = {"homeId": home.get("homeId")}
-        data = await self.request("getIndependentDevices", payload)
+        data = await self.request("getIndependentDevices2020", payload)
         if data is None:
             return
-        heater_data = data.get("deviceInfo", [])
-        if not heater_data:
+        dev_data = data.get("deviceInfo", [])
+        if not dev_data:
             return
-        for _heater in heater_data:
-            _id = _heater.get("deviceId")
-            heater = self.heaters.get(_id, Heater())
-            heater.device_id = _id
-            set_heater_values(_heater, heater)
-            self.heaters[_id] = heater
-            if heater.home_id is None:
-                heater.home_id = home.get("homeId")
+        for dev in dev_data:
+            _id = dev.get("deviceId")
+            print(dev)
+
+            if dev["subDomainId"] in (6933,):
+                self.sensors[_id] = Sensor.init_from_response(dev)
+            else:
+                heater = self.heaters.get(_id, Heater())
+                heater.device_id = _id
+                set_heater_values(dev, heater)
+                self.heaters[_id] = heater
+                if heater.home_id is None:
+                    heater.home_id = home.get("homeId")
 
     async def _update_heater_data(self, heater):
         payload = {"deviceId": heater.device_id}
@@ -401,76 +415,38 @@ class Mill:
         self.heaters[heater.device_id] = heater
 
     async def _update_consumption(self, heater):
-        if (
-            heater.last_consumption_update
-            and (dt.datetime.now() - heater.last_consumption_update)
-            < MIN_TIME_BETWEEN_STATS_UPDATES
-        ):
+        now = dt.datetime.now()
+        if heater.last_consumption_update and (
+            now - heater.last_consumption_update
+        ) < MIN_TIME_BETWEEN_STATS_UPDATES + dt.timedelta(random.randint(0, 60)):
             return
 
-        async with self._lock_cons_data:
-            cons = await self.request_stats(
-                "statisticDeviceForAndroid",
-                {
-                    "dateType": 1,
-                    "dateStr": dt.datetime.now().strftime("%Y-%m-%d"),
-                    "timeZone": "GMT+02:00",
-                    "haveSensor": 1,
-                    "deviceId": heater.device_id,
-                },
+        payload = {
+            "dateType": None,
+            "dateStr": now.strftime("%Y-%m-%d"),
+            "timeZone": "GMT+02:00",
+            "haveSensor": 1,
+            "deviceId": heater.device_id,
+        }
+
+        tasks = []
+        for date_type in [0, 3]:
+            payload["dateType"] = date_type
+            tasks.append(
+                self.request_stats(
+                    "statisticDeviceForAndroid",
+                    payload,
+                )
             )
 
-            if cons is None or cons.get("code") != 0:
-                return
+        (cons0, cons3) = await asyncio.gather(*tasks)
 
-            heater.day_consumption = float(cons.get("valueTotal"))
+        if cons0 is not None and cons0.get("code") == 0:
+            heater.day_consumption = float(cons0.get("valueTotal"))
+            heater.last_consumption_update = now
 
-            cons = await self.request_stats(
-                "statisticDeviceForAndroid",
-                {
-                    "dateType": 3,
-                    "dateStr": dt.datetime.now().strftime("%Y-%m-%d"),
-                    "timeZone": "GMT+02:00",
-                    "haveSensor": 1,
-                    "deviceId": heater.device_id,
-                },
-            )
-
-            if cons is None or cons.get("code") != 0:
-                return
-
-            heater.year_consumption = float(cons.get("valueTotal"))
-            heater.last_consumption_update = dt.datetime.now()
-
-    async def throttle_update_heaters(self):
-        """Throttle update device."""
-        if (
-            self._throttle_time is not None
-            and dt.datetime.now() - self._throttle_time < MIN_TIME_BETWEEN_UPDATES
-        ):
-            return
-        self._throttle_time = dt.datetime.now()
-        await self.update_heaters()
-
-    async def throttle_update_all_heaters(self):
-        """Throttle update all devices and rooms."""
-        if (
-            self._throttle_all_time is not None
-            and dt.datetime.now() - self._throttle_all_time < MIN_TIME_BETWEEN_UPDATES
-        ):
-            return
-        self._throttle_all_time = dt.datetime.now()
-        await self.find_all_heaters()
-
-    async def update_device(self, device_id):
-        """Update device."""
-        await self.throttle_update_heaters()
-        return self.heaters.get(device_id)
-
-    async def update_room(self, room_id):
-        """Update room."""
-        await self.throttle_update_all_heaters()
-        return self.rooms.get(room_id)
+        if cons3 is not None and cons3.get("code") == 0:
+            heater.year_consumption = float(cons3.get("valueTotal"))
 
     async def heater_control(self, device_id, fan_status=None, power_status=None):
         """Set heater temps."""
@@ -507,11 +483,6 @@ class Mill:
         }
         await self.request("changeDeviceInfo", payload)
 
-    async def find_all_heaters(self):
-        """Find all heaters."""
-        await self.update_rooms()
-        await self.update_heaters()
-
 
 class Room:
     """Representation of room."""
@@ -568,6 +539,11 @@ class Heater:
             863,
         ]
 
+    @property
+    def generation(self):
+        """Check if heater is gen 1."""
+        return 1 if self.is_gen1 else 2
+
     def __repr__(self):
         items = (f"{k}={v}" for k, v in self.__dict__.items())
         return f"{self.__class__.__name__}({', '.join(items)})"
@@ -575,9 +551,13 @@ class Heater:
 
 def set_heater_values(heater_data, heater):
     """Set heater values from heater data"""
-    heater.current_temp = heater_data.get("currentTemp")
-    if heater.current_temp > 90.0:
+    try:
+        heater.current_temp = float(heater_data.get("currentTemp"))
+    except ValueError:
         heater.current_temp = None
+    else:
+        if heater.current_temp > 90.0:
+            heater.current_temp = None
     heater.device_status = heater_data.get("deviceStatus")
     heater.available = heater.device_status == 0
     heater.name = heater_data.get("deviceName")
@@ -618,3 +598,34 @@ def set_heater_values(heater_data, heater):
         )
     except ValueError:
         pass
+
+
+@dataclass
+class Sensor:
+    """Representation of sensor."""
+
+    name: str
+    device_id: int
+    current_temp: float
+    humidity: float
+    tvoc: float
+    eco2: float
+    battery: float
+
+    @property
+    def available(self):
+        """Available."""
+        return True
+
+    @classmethod
+    def init_from_response(cls, response):
+        """Class method."""
+        return cls(
+            response.get("deviceName"),
+            response.get("deviceId"),
+            response.get("currentTemp"),
+            response.get("humidity"),
+            response.get("tvoc"),
+            response.get("eco3"),
+            response.get("batteryPer"),
+        )
